#!/bin/bash

source ./config.sh

SEALPIR_DIRECTORY="SealPIR"

if [ -d $SEALPIR_DIRECTORY ]; then
	echo "Folder Found for SealPIR!"
else
	echo "Error : SealPIR Folder not found"
	exit
fi

cd $SEALPIR_DIRECTORY
mkdir -p "Output_logs"
echo

echo "Compiling SealPIR..."
make>make_log_sealpir.txt
echo

no_of_blocks=$BLOCKS_START

# SealPIR (v2.1) cannot handle larger blocksizes than 3072 B as we detail in our paper
# Hence we use 3072 and scale the timing benchmarks by K to match up to the 16384 B that
# we use for all the other PIR schemes.

SEALPIR_BLOCK_SIZE=3072
K=$(( (BLOCK_SIZE-1)/ SEALPIR_BLOCK_SIZE + 1)) 

while [ "$no_of_blocks" -le $BLOCKS_STOP ]
  do
   
  echo "Running experiment with number of items n = "$no_of_blocks
  INTERMEDIATE_LOG="Output_logs/SEAL_"$no_of_blocks"_"$BLOCK_SIZE

  if [ $no_of_blocks -gt 665 ]; then
    D=2
  fi

  echo "Levels of recursion d = "$D
  SEALPIR_COMMAND="./bin/sealpir "$no_of_blocks" "$SEALPIR_BLOCK_SIZE" "$D" "$NUM_OF_REQUESTS

  echo "Executing :"
  echo $SEALPIR_COMMAND 
  $SEALPIR_COMMAND > $SEALPIR_EXEC_LOG
  echo

  no_of_blocks=$((no_of_blocks*8/5))
  echo $no_of_blocks

  #Parse SealPIR exec log 
  PY_COMMAND="python ./../extract_sealpir_results.py "$SEALPIR_EXEC_LOG" "$K" "$INTERMEDIATE_LOG" "$SEALPIR_RESULT_LOG
  echo $PY_COMMAND
  $PY_COMMAND

  done



